#!/usr/bin/env node
var shell = require('shelljs');
const semver = require('semver');
const updateNotifier = require('update-notifier');
const updateTagPackage = require('./package.json');
const prompts = require('prompts');

const [, , ...args] = process.argv;
const dryRun = args[0] == '--dry-run';

(async function() {
    function clearScreen() {
        process.stdout.write('\033c');
    }

    function disableShellOutput() {
        shell.config.silent = true;
    }

    function enableShellOutput() {
        shell.config.silent = false;
    }

    function quitIfGitIsMissing() {
        if (!shell.which('git')) {
            shell.echo('Error: This script requires git');
            shell.exit(1);
        }
    }

    function quitIfNoGitRepo() {
        if (shell.exec('git rev-parse --is-inside-work-tree').code !== 0) {
            shell.echo('Error: Not inside a git directory');
            shell.exit(1);
        }
    }

    function showMessageIfDryRun() {
        if (dryRun) {
            console.log('DRY RUN - NOTHING WILL BE PUSHED');
        }
    }

    function getLatestTag() {
        return shell.exec('git describe --tags $(git rev-list --tags --max-count=1)').stdout.trim();
    }

    async function askToPush(tag) {
        const question = {
            type: 'select',
            name: 'value',
            message: `Add tag [${tag}] and push to origin?`,
            choices: [{ title: `yes`, value: true }, { title: `no`, value: false }],
            initial: 1
        };

        let response = await prompts(question);
        if (response.value) {
            push(tag);
        }
    }

    function push(tag) {
        enableShellOutput();
        if (!dryRun) {
            shell.exec(`git tag ${tag}`);
            shell.exec(`git push`);
            shell.exec(`git push --tags`);
        } else {
            console.log('NOT PUSHING BECAUSE OF: --dry-run');
        }
        process.exit();
    }

    async function showMenu(latestTag) {
        const prerelease = semver.inc(latestTag, 'prerelease', 'RC');
        const patch = semver.inc(latestTag, 'patch');
        const minor = semver.inc(latestTag, 'minor');
        const major = semver.inc(latestTag, 'major');

        console.log(`  Latest tag \t [${latestTag}]`);

        const question = {
            type: 'select',
            name: 'value',
            message: 'Select new tag:',
            choices: [
                { title: `prerelease \t [${prerelease}]`, value: 'prerelease' },
                { title: `patch \t [${patch}]`, value: patch },
                { title: `minor \t [${minor}]`, value: minor },
                { title: `major \t [${major}]`, value: major },
                { title: `quit`, value: 'quit' }
            ],
            initial: 4
        };
        let response = await prompts(question);

        if (response.value === 'quit') {
            process.exit();
        }

        await askToPush(response.value);
    }

    clearScreen();
    updateNotifier({ pkg: updateTagPackage }).notify({ defer: false });
    disableShellOutput();
    quitIfGitIsMissing();
    quitIfNoGitRepo();
    showMessageIfDryRun();
    const latestTag = getLatestTag();
    await showMenu(latestTag);
})();
